package lightcontainer.utils;

import java.nio.charset.StandardCharsets;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.util.Base64;

/**
 * Permet de hasher du texte
 */
public class ShaHasher {
    /**
     * Mot de passe non-hashé
     */
    private final String password;

    /**
     * Sallage appliqué sur le mot de passe précédement généré
     */
    private byte[] salt;

    public ShaHasher(String password) {
        this.password = password;
    }

    /**
     * Permet de demander un nouvel hashage sur le mot de passe
     *
     * @return Mot de passe hashé
     * <p>
     * Source : https://www.javaguides.net/2020/02/java-sha-384-hash-with-salt-example.html
     */
    public String nextHashing() {
        this.salt = generateSalt();

        return fromSalt(this.salt);
    }

    public String fromSalt(byte[] passwordSalt) {
        String generatedPassword = null;
        try {
            MessageDigest md = MessageDigest.getInstance("SHA-384");
            md.update(passwordSalt);
            byte[] bytes = md.digest(password.getBytes(StandardCharsets.UTF_8));
            StringBuilder sb = new StringBuilder();
            for (byte aByte : bytes) {
                sb.append(Integer.toString((aByte & 0xff) + 0x100, 16).substring(1));
            }
            generatedPassword = sb.toString();
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        }
        return generatedPassword;
    }


    private byte[] generateSalt() {
        this.salt = new byte[16];
        SecureRandom random = new SecureRandom();

        random.nextBytes(salt);
        return salt;
    }


    public String getSalt() {
        Base64.Encoder b64Encoder = Base64.getEncoder();
        return b64Encoder.encodeToString(this.salt);
    }

    public byte[] saltToByte(String salt) {
        Base64.Decoder b64Decoder = Base64.getDecoder();
        return b64Decoder.decode(salt);
    }

}
